@@ -32,14 +32,13 @@ use crate::openpgp::policy::Policy;
 use crate::{
     Config,
     parse_armor_kind,
-    create_or_stdout_pgp,
 };
 
 #[cfg(feature = "autocrypt")]
 pub mod autocrypt;
 pub mod decrypt;
 pub use self::decrypt::decrypt;
-mod sign;
+pub mod sign;
 pub use self::sign::sign;
 pub mod dump;
 pub use self::dump::dump;
@@ -265,13 +264,15 @@ impl<'a> VHelper<'a> {
     }
 
     fn print_sigs(&mut self, results: &[VerificationResult]) {
+        use crate::print_error_chain;
         use self::VerificationError::*;
         for result in results {
             let (issuer, level) = match result {
                 Ok(GoodChecksum { sig, ka, .. }) =>
                     (ka.key().keyid(), sig.level()),
                 Err(MalformedSignature { error, .. }) => {
-                    eprintln!("Malformed signature: {}", error);
+                    eprintln!("Malformed signature:");
+                    print_error_chain(error);
                     self.broken_signatures += 1;
                     continue;
                 },
@@ -288,14 +289,16 @@ impl<'a> VHelper<'a> {
                     continue;
                 },
                 Err(UnboundKey { cert, error, .. }) => {
-                    eprintln!("Signing key on {} is not bound: {}",
-                              cert.fingerprint(), error);
+                    eprintln!("Signing key on {} is not bound:",
+                              cert.fingerprint());
+                    print_error_chain(error);
                     self.bad_checksums += 1;
                     continue;
                 },
                 Err(BadKey { ka, error, .. }) => {
-                    eprintln!("Signing key on {} is bad: {}",
-                              ka.cert().fingerprint(), error);
+                    eprintln!("Signing key on {} is bad:",
+                              ka.cert().fingerprint());
+                    print_error_chain(error);
                     self.bad_checksums += 1;
                     continue;
                 },
@@ -305,8 +308,9 @@ impl<'a> VHelper<'a> {
                         0 => "checksum".into(),
                         n => format!("level {} notarizing checksum", n),
                     };
-                    eprintln!("Error verifying {} from {}: {}",
-                              what, issuer, error);
+                    eprintln!("Error verifying {} from {}:",
+                              what, issuer);
+                    print_error_chain(error);
                     self.bad_checksums += 1;
                     continue;
                 }
@@ -454,21 +458,22 @@ pub fn join(config: Config, m: &clap::ArgMatches)
     let mut sink =
         if m.is_present("binary") {
             // No need for any auto-detection.
-            Some(create_or_stdout_pgp(output, config.force,
-                                      true, // Binary.
-                                      armor::Kind::File)?)
+            Some(config.create_or_stdout_pgp(output,
+                                             true, // Binary.
+                                             armor::Kind::File)?)
         } else if let Some(kind) = kind {
-            Some(create_or_stdout_pgp(output, config.force,
-                                      false, // Armored.
-                                      kind)?)
+            Some(config.create_or_stdout_pgp(output,
+                                             false, // Armored.
+                                             kind)?)
         } else {
             None // Defer.
         };
 
     /// Writes a bit-accurate copy of all top-level packets in PPR to
     /// OUTPUT.
-    fn copy(mut ppr: PacketParserResult,
-            output: Option<&str>, force: bool,
+    fn copy(config: &Config,
+            mut ppr: PacketParserResult,
+            output: Option<&str>,
             sink: &mut Option<Message>)
             -> Result<()> {
         while let PacketParserResult::Some(pp) = ppr {
@@ -483,9 +488,9 @@ pub fn join(config: Config, m: &clap::ArgMatches)
                     _ => armor::Kind::File,
                 };
 
-                *sink = Some(create_or_stdout_pgp(output, force,
-                                                  false, // Armored.
-                                                  kind)?);
+                *sink = Some(config.create_or_stdout_pgp(output,
+                                                         false, // Armored.
+                                                         kind)?);
             }
 
             // We (ab)use the mapping feature to create byte-accurate
@@ -505,13 +510,13 @@ pub fn join(config: Config, m: &clap::ArgMatches)
             let ppr =
                 openpgp::parse::PacketParserBuilder::from_file(name)?
                 .map(true).build()?;
-            copy(ppr, output, config.force, &mut sink)?;
+            copy(&config, ppr, output, &mut sink)?;
         }
     } else {
         let ppr =
             openpgp::parse::PacketParserBuilder::from_reader(io::stdin())?
             .map(true).build()?;
-        copy(ppr, output, config.force, &mut sink)?;
+        copy(&config, ppr, output, &mut sink)?;
     }
 
     sink.unwrap().finalize()?;
