@@ -38,6 +38,7 @@ fn open_or_stdin(f: Option<&str>)
     }
 }
 
+#[deprecated(note = "Use the appropriate function on Config instead")]
 fn create_or_stdout(f: Option<&str>, force: bool)
     -> Result<Box<dyn io::Write + Sync + Send>> {
     match f {
@@ -54,24 +55,13 @@ fn create_or_stdout(f: Option<&str>, force: bool)
                             .context("Failed to create output file")?))
             } else {
                 Err(anyhow::anyhow!(
-                    format!("File {:?} exists, use --force to overwrite", p)))
+                    format!("File {:?} exists, use \"sq --force ...\" to \
+                             overwrite", p)))
             }
         }
     }
 }
 
-fn create_or_stdout_pgp<'a>(f: Option<&str>, force: bool,
-                            binary: bool, kind: armor::Kind)
-                            -> Result<Message<'a>>
-{
-    let sink = create_or_stdout(f, force)?;
-    let mut message = Message::new(sink);
-    if ! binary {
-        message = Armorer::new(message).kind(kind).build()?;
-    }
-    Ok(message)
-}
-
 const SECONDS_IN_DAY : u64 = 24 * 60 * 60;
 const SECONDS_IN_YEAR : u64 =
     // Average number of days in a year.
@@ -353,18 +343,55 @@ fn emit_unstable_cli_warning() {
 pub struct Config<'a> {
     force: bool,
     policy: P<'a>,
+    /// Have we emitted the warning yet?
+    unstable_cli_warning_emitted: bool,
+}
+
+impl Config<'_> {
+    /// Opens the file (or stdout) for writing data that is safe for
+    /// non-interactive use.
+    ///
+    /// This is suitable for any kind of OpenPGP data, or decrypted or
+    /// authenticated payloads.
+    fn create_or_stdout_safe(&self, f: Option<&str>)
+                             -> Result<Box<dyn io::Write + Sync + Send>> {
+        #[allow(deprecated)]
+        create_or_stdout(f, self.force)
+    }
+
+    /// Opens the file (or stdout) for writing data that is NOT safe
+    /// for non-interactive use.
+    ///
+    /// If our heuristic detects non-interactive use, we will emit a
+    /// warning.
+    fn create_or_stdout_unsafe(&mut self, f: Option<&str>)
+                               -> Result<Box<dyn io::Write + Sync + Send>> {
+        if ! self.unstable_cli_warning_emitted {
+            emit_unstable_cli_warning();
+            self.unstable_cli_warning_emitted = true;
+        }
+        #[allow(deprecated)]
+        create_or_stdout(f, self.force)
+    }
+
+    /// Opens the file (or stdout) for writing data that is safe for
+    /// non-interactive use because it is an OpenPGP data stream.
+    fn create_or_stdout_pgp<'a>(&self, f: Option<&str>,
+                                binary: bool, kind: armor::Kind)
+                                -> Result<Message<'a>> {
+        let sink = self.create_or_stdout_safe(f)?;
+        let mut message = Message::new(sink);
+        if ! binary {
+            message = Armorer::new(message).kind(kind).build()?;
+        }
+        Ok(message)
+    }
 }
 
 fn main() -> Result<()> {
     let policy = &mut P::new();
 
-    // XXX: Compat with sequoia-openpgp 1.0.0:
-    use openpgp::packet::signature::subpacket::SubpacketTag;
-    policy.accept_critical_subpacket(SubpacketTag::TrustSignature);
-    policy.accept_critical_subpacket(SubpacketTag::RegularExpression);
-
     let matches = sq_cli::build().get_matches();
-    emit_unstable_cli_warning();
 
     let known_notations: Vec<&str> = matches.values_of("known-notation")
         .unwrap_or_default()
@@ -373,20 +400,35 @@ fn main() -> Result<()> {
 
     let force = matches.is_present("force");
 
-    let config = Config {
+    let mut config = Config {
         force,
         policy: policy.clone(),
+        unstable_cli_warning_emitted: false,
     };
 
     match matches.subcommand() {
         ("decrypt",  Some(m)) => {
             let mut input = open_or_stdin(m.value_of("input"))?;
-            let mut output = create_or_stdout(m.value_of("output"), force)?;
-            let signatures: usize =
-                m.value_of("signatures").expect("has a default").parse()?;
+            let mut output =
+                config.create_or_stdout_safe(m.value_of("output"))?;
             let certs = m.values_of("sender-cert-file")
                 .map(load_certs)
                 .unwrap_or(Ok(vec![]))?;
+            // Fancy default for --signatures.  If you change this,
+            // also change the description in the CLI definition.
+            let signatures: usize =
+                if let Some(n) = m.value_of("signatures") {
+                    n.parse()?
+                } else if certs.is_empty() {
+                    // No certs are given for verification, use 0 as
+                    // threshold so we handle only-encrypted messages
+                    // gracefully.
+                    0
+                } else {
+                    // At least one cert given, expect at least one
+                    // valid signature.
+                    1
+                };
             let secrets = m.values_of("secret-key-file")
                 .map(load_keys)
                 .unwrap_or(Ok(vec![]))?;
@@ -402,9 +444,9 @@ fn main() -> Result<()> {
                 .unwrap_or(Ok(vec![]))?;
             let mut input = open_or_stdin(m.value_of("input"))?;
             let output =
-                create_or_stdout_pgp(m.value_of("output"), force,
-                                     m.is_present("binary"),
-                                     armor::Kind::Message)?;
+                config.create_or_stdout_pgp(m.value_of("output"),
+                                            m.is_present("binary"),
+                                            armor::Kind::Message)?;
             let additional_secrets = m.values_of("signer-key-file")
                 .map(load_keys)
                 .unwrap_or(Ok(vec![]))?;
@@ -451,42 +493,47 @@ fn main() -> Result<()> {
             } else {
                 None
             };
+            // Each --notation takes two values.  The iterator
+            // returns them one at a time, however.
+            let mut notations: Vec<(bool, NotationData)> = Vec::new();
+            if let Some(mut n) = m.values_of("notation") {
+                while let Some(name) = n.next() {
+                    let value = n.next().unwrap();
+
+                    let (critical, name) = if name.len() > 0
+                        && Some('!') == name.chars().next()
+                    {
+                        (true, &name[1..])
+                    } else {
+                        (false, name)
+                    };
+
+                    notations.push(
+                        (critical,
+                         NotationData::new(
+                             name, value,
+                             NotationDataFlags::empty().set_human_readable())));
+                }
+            }
+
             if let Some(merge) = m.value_of("merge") {
-                let output = create_or_stdout_pgp(output, force, binary,
-                                                  armor::Kind::Message)?;
+                let output = config.create_or_stdout_pgp(output, binary,
+                                                         armor::Kind::Message)?;
                 let mut input2 = open_or_stdin(Some(merge))?;
                 commands::merge_signatures(&mut input, &mut input2, output)?;
+            } else if m.is_present("clearsign") {
+                let output = config.create_or_stdout_safe(output)?;
+                commands::sign::clearsign(config, input, output, secrets,
+                                          time, &notations)?;
             } else {
-                // Each --notation takes two values.  The iterator
-                // returns them one at a time, however.
-                let mut notations: Vec<(bool, NotationData)> = Vec::new();
-                if let Some(mut n) = m.values_of("notation") {
-                    while let Some(name) = n.next() {
-                        let value = n.next().unwrap();
-
-                        let (critical, name) = if name.len() > 0
-                            && Some('!') == name.chars().next()
-                        {
-                            (true, &name[1..])
-                        } else {
-                            (false, name)
-                        };
-
-                        notations.push(
-                            (critical,
-                             NotationData::new(
-                                 name, value,
-                                 NotationDataFlags::empty().set_human_readable())));
-                    }
-                }
-
-                commands::sign(policy, &mut input, output, secrets, detached,
-                               binary, append, notarize, time, &notations, force)?;
+                commands::sign(config, &mut input, output, secrets, detached,
+                               binary, append, notarize, time, &notations)?;
             }
         },
         ("verify",  Some(m)) => {
             let mut input = open_or_stdin(m.value_of("input"))?;
-            let mut output = create_or_stdout(m.value_of("output"), force)?;
+            let mut output =
+                config.create_or_stdout_safe(m.value_of("output"))?;
             let mut detached = if let Some(f) = m.value_of("detached") {
                 Some(File::open(f)?)
             } else {
@@ -523,7 +570,7 @@ fn main() -> Result<()> {
             {
                 // It is already armored and has the correct kind.
                 let mut output =
-                    create_or_stdout(m.value_of("output"), force)?;
+                    config.create_or_stdout_safe(m.value_of("output"))?;
                 io::copy(&mut input, &mut output)?;
                 return Ok(());
             }
@@ -538,8 +585,8 @@ fn main() -> Result<()> {
             let want_kind = want_kind.expect("given or detected");
 
             let mut output =
-                create_or_stdout_pgp(m.value_of("output"), force,
-                                     false, want_kind)?;
+                config.create_or_stdout_pgp(m.value_of("output"),
+                                            false, want_kind)?;
 
             if already_armored {
                 // Dearmor and copy to change the type.
@@ -554,7 +601,8 @@ fn main() -> Result<()> {
         },
         ("dearmor",  Some(m)) => {
             let mut input = open_or_stdin(m.value_of("input"))?;
-            let mut output = create_or_stdout(m.value_of("output"), force)?;
+            let mut output =
+                config.create_or_stdout_safe(m.value_of("output"))?;
             let mut filter = armor::Reader::new(&mut input, None);
             io::copy(&mut filter, &mut output)?;
         },
@@ -562,16 +610,18 @@ fn main() -> Result<()> {
         ("autocrypt", Some(m)) => commands::autocrypt::dispatch(config, m)?,
 
         ("inspect",  Some(m)) => {
-            let mut output = create_or_stdout(m.value_of("output"), force)?;
+            let mut output =
+                config.create_or_stdout_unsafe(m.value_of("output"))?;
             commands::inspect(m, policy, &mut output)?;
         },
 
-        ("keyring", Some(m)) => commands::keyring::dispatch(m, force)?,
+        ("keyring", Some(m)) => commands::keyring::dispatch(config, m)?,
 
         ("packet", Some(m)) => match m.subcommand() {
             ("dump",  Some(m)) => {
                 let mut input = open_or_stdin(m.value_of("input"))?;
-                let mut output = create_or_stdout(m.value_of("output"), force)?;
+                let mut output =
+                    config.create_or_stdout_unsafe(m.value_of("output"))?;
                 let session_key: Option<openpgp::crypto::SessionKey> =
                     if let Some(sk) = m.value_of("session-key") {
                         Some(hex::decode_pretty(sk)?.into())
@@ -587,9 +637,9 @@ fn main() -> Result<()> {
             ("decrypt",  Some(m)) => {
                 let mut input = open_or_stdin(m.value_of("input"))?;
                 let mut output =
-                    create_or_stdout_pgp(m.value_of("output"), force,
-                                         m.is_present("binary"),
-                                         armor::Kind::Message)?;
+                    config.create_or_stdout_pgp(m.value_of("output"),
+                                                m.is_present("binary"),
+                                                armor::Kind::Message)?;
                 let secrets = m.values_of("secret-key-file")
                     .map(load_keys)
                     .unwrap_or(Ok(vec![]))?;
@@ -712,3 +762,9 @@ fn test_parse_iso8601() {
     parse_iso8601("2017031", z).unwrap();
     // parse_iso8601("2017", z).unwrap(); // ditto
 }
+
+/// Prints the error and causes, if any.
+pub fn print_error_chain(err: &anyhow::Error) {
+    eprintln!("           {}", err);
+    err.chain().skip(1).for_each(|cause| eprintln!("  because: {}", cause));
+}
